from nuclia.sdk.kb import NucliaKB


def test_extract_strategies(testing_config):
    nkb = NucliaKB()
    # preventive clean up
    for id in nkb.extract_strategies.list().keys():
        nkb.extract_strategies.delete(id=id)

    # tests
    nkb.extract_strategies.add(config={"name": "strategy1", "vllm_config": {}})
    all = nkb.extract_strategies.list()
    assert len(all.keys()) == 1
    nkb.extract_strategies.delete(id=list(all.keys())[0])
    all = nkb.extract_strategies.list()
    assert len(all.keys()) == 0



import logging
import sys
from importlib.metadata import version

import fire  # type: ignore
from nucliadb_sdk import exceptions

from nuclia.data import get_auth
from nuclia.exceptions import NeedUserToken, UserTokenExpired
from nuclia.lib.utils import serialize
from nuclia.sdk.accounts import NucliaAccounts
from nuclia.sdk.backup import NucliaBackup
from nuclia.sdk.kb import NucliaKB
from nuclia.sdk.kbs import NucliaKBS
from nuclia.sdk.logger import logger
from nuclia.sdk.nua import NucliaNUA
from nuclia.sdk.zones import NucliaZones

from .utils import CustomFormatter


class NucliaCLI(object):
    def __init__(self):
        self.auth = get_auth()
        self.accounts = NucliaAccounts()
        self.zones = NucliaZones()
        self.kbs = self.knowledgeboxes = NucliaKBS()
        self.kb = self.knowledgebox = NucliaKB()
        self.nua = NucliaNUA()
        self.backup = NucliaBackup()

    def version(self):
        """Print the version of the CLI"""
        return version("nuclia")


def run():
    logging.basicConfig(level=logging.INFO, handlers=[])
    ch = logging.StreamHandler()
    ch.setLevel(logging.INFO)
    ch.setFormatter(CustomFormatter())
    logger.addHandler(ch)

    try:
        fire.Fire(NucliaCLI, serialize=serialize)
    except exceptions.AuthError:
        handleAuthError()
    except NeedUserToken:
        handleAuthError()
    except UserTokenExpired:
        handleAuthError()


def handleAuthError():
    logger.error("Login required.")
    logger.info("Run `nuclia auth login` to login.")
    sys.exit(1)



from nuclia.sdk.kb import NucliaKB
from nucliadb_sdk.v2.exceptions import NotFoundError

from time import sleep


def test_graph(testing_config):
    nkb = NucliaKB()

    # ensure KB is clean
    try:
        nkb.delete_graph(slug="graph1")
    except NotFoundError:
        pass

    nkb.add_graph(
        slug="graph1",
        graph=[
            {
                "source": {"group": "People", "value": "Alice"},
                "destination": {"group": "People", "value": "Bob"},
                "label": "is friend of",
            },
            {
                "source": {"group": "People", "value": "Alice"},
                "destination": {"group": "City", "value": "Toulouse"},
                "label": "lives in",
            },
            {
                "source": {"group": "People", "value": "Bob"},
                "destination": {"group": "Food", "value": "Cheese"},
                "label": "eat",
            },
        ],
    )
    relations = nkb.get_graph(slug="graph1")
    assert len(relations) == 3

    # XXX: sometimes, nucliadb needs some more time to index the graph. We retry
    # some times and fail if that keeps hapenning
    RETRIES = 5
    DELAY = 0.5
    for _ in range(RETRIES):
        paths = nkb.search.graph(
            query={"query": {"prop": "path", "source": {"value": "Alice"}}}
        )
        try:
            assert len(paths.paths) == 2
        except AssertionError:
            # wait for a bit before retrying
            print("Graph was not indexed yet, waiting a bit...")
            sleep(DELAY)
        else:
            break
    assert len(paths.paths) == 2

    nkb.update_graph(
        slug="graph1",
        graph=[
            {
                "source": {"group": "People", "value": "Victor"},
                "destination": {"group": "Food", "value": "Cheese"},
                "label": "eat",
            },
        ],
    )
    relations = nkb.get_graph(slug="graph1")
    assert len(relations) == 4

    for _ in range(RETRIES):
        paths = nkb.search.graph(
            query={"query": {"prop": "path", "source": {"value": "Victor"}}}
        )
        try:
            assert len(paths.paths) == 1
        except AssertionError:
            print("Graph was not indexed yet, waiting a bit...")
            sleep(DELAY)
        else:
            break
    assert len(paths.paths) == 1

    nkb.delete_graph(slug="graph1")
    try:
        sleep(0.5)
        nkb.get_graph(slug="graph1")
        assert False
    except NotFoundError:
        assert True



from nuclia.data import get_auth
from nuclia.sdk.auth import NucliaAuth


class NucliaZones:
    @property
    def _auth(self) -> NucliaAuth:
        return get_auth()

    def list(self):
        return self._auth.zones()

    def default(self, zone: str):
        self._auth._config.set_default_zone(zone)
        self._auth._config.save()
